## Copyright (C) 2024 David Legland
## All rights reserved.
## 
## Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
## 
##     1 Redistributions of source code must retain the above copyright notice,
##       this list of conditions and the following disclaimer.
##     2 Redistributions in binary form must reproduce the above copyright
##       notice, this list of conditions and the following disclaimer in the
##       documentation and/or other materials provided with the distribution.
## 
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ''AS IS''
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
## IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
## ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
## ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
## DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
## SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
## CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
## OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
## 
## The views and conclusions contained in the software and documentation are
## those of the authors and should not be interpreted as representing official
## policies, either expressed or implied, of the copyright holders.

function polyShape = polygonToPolyshape(poly, varargin)
%POLYGONTOPOLYSHAPE Convert a matGeom polygon to a MATLAB polyshape object.
%
%   POLYSHAPE = polygonToPolyshape(POLY)
%
%   Example
%     poly = [0 0; 1 0; 1 1; 0 1];
%     polyShape = polygonToPolyshape(poly);
%     figure('color','w')
%     axis equal tight; hold on; xlabel('x'); ylabel('y')
%     plot(polyShape)
%
%   See also 
%   rowToPolygon

% ------
% Author: oqilipo
% E-mail: N/A
% Created: 2022-12-31, using MATLAB 9.13.0.2080170 (R2022b) Update 1
% Copyright 2022-2023

parser = inputParser;
logParValidFunc = @(x) (islogical(x) || isequal(x,1) || isequal(x,0));
addParameter(parser,'debugVisualization', false, logParValidFunc);
parse(parser, varargin{:});
debugVisu = parser.Results.debugVisualization;

polyShape = parsePolygon(poly, 'polyshape');

if debugVisu
    figure('color','w','numbertitle','off', ...
        'name', ['Debug Figure: ' mfilename ...
        '.m: MATLAB polyshape object']);
    axis equal tight; hold on; xlabel('x'); ylabel('y')
    plot(polyShape)
end

end
