## Copyright (C) 2024 David Legland
## All rights reserved.
## 
## Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
## 
##     1 Redistributions of source code must retain the above copyright notice,
##       this list of conditions and the following disclaimer.
##     2 Redistributions in binary form must reproduce the above copyright
##       notice, this list of conditions and the following disclaimer in the
##       documentation and/or other materials provided with the distribution.
## 
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ''AS IS''
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
## IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
## ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
## ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
## DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
## SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
## CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
## OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
## OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
## 
## The views and conclusions contained in the software and documentation are
## those of the authors and should not be interpreted as representing official
## policies, either expressed or implied, of the copyright holders.

function faces2 = removeDuplicateFaces(faces, varargin)
%REMOVEDUPLICATEFACES Remove duplicate faces in a face array.
%
%   [V, F] = removeDuplicateFaces(V, F)
%
%   Example
%     faces = [1 2 3;2 3 4;3 4 5;2 3 4];
%     removeDuplicateFaces(faces)
%     ans =
%         1 2 3
%         2 3 4
%         2 3 5
%
%   See also 
%     ensureManifoldMesh

% ------
% Author: David Legland
% E-mail: david.legland@inrae.fr
% Created: 2019-01-08, using Matlab 8.6.0.267246 (R2015b)
% Copyright 2019-2023 INRA - Cepia Software Platform

nFaces = size(faces, 1);

removeFlag = false(nFaces, 1);
for iFace = 1:nFaces
    if removeFlag(iFace)
        continue;
    end
    
    face = faces(iFace, :);
    
    inds = find(sum(ismember(faces, face), 2) == 3);
    inds(inds <= iFace) = [];
    
    if ~isempty(inds)
        removeFlag(inds) = true;
    end
end

faces2 = faces(~removeFlag, :);
